#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <mac/All.h>
#include <mac/MACLib.h>
#include <mac/CharacterHelper.h>

#define BLOCKS_PER_DECODE 9216

int dsp_fd;
int channels;
short *outbuf;
char *filename;

int main( int argc, char **argv ) {
  int rc, idx;

  if ( argc < 2 ) {
  	puts( "Usage: wpkplay <filename.wv> ..." );
	return 1;
  }

  dsp_fd = open( "/dev/dsp", O_WRONLY );
  if ( dsp_fd == -1 ) {
	perror( "open /dev/dsp" );
	return 1;
  }

  for ( idx = 1; idx < argc; idx++ ) {
	filename = argv[idx];

	printf( "Playing stream from %s...\n", filename );

	IAPEDecompress *apedec;
	int errcode = 0;
	apedec = CreateIAPEDecompress( GetUTF16FromANSI( filename ), &errcode );
	if ( !apedec ) {
		puts( "failed to open file" );
		continue;
	}

	int blk_align = apedec->GetInfo(APE_INFO_BLOCK_ALIGN);
	outbuf = (short*) malloc( blk_align * BLOCKS_PER_DECODE );

	int val = apedec->GetInfo( APE_INFO_BITS_PER_SAMPLE );
	if ( val == 16 )
		val = AFMT_S16_LE;
	else {
	  	printf( "%d bits per sample format not supported\n", val );
		exit( 1 );
	}

	ioctl( dsp_fd, SNDCTL_DSP_SETFMT, &val );

	val = apedec->GetInfo( APE_INFO_CHANNELS );
	channels = val;
	if ( val == 1 )
	  	val = 0;
	else if ( val == 2 )
		val = 1;
	else {
  		printf( "%d channels format not supported\n", val );
		exit( 1 );
	}

	ioctl( dsp_fd, SNDCTL_DSP_STEREO, &val );

	val = apedec->GetInfo( APE_INFO_SAMPLE_RATE );
	ioctl( dsp_fd, SNDCTL_DSP_SPEED, &val );

	int blk_count, tot_blk;

	tot_blk = apedec->GetInfo( APE_DECOMPRESS_TOTAL_BLOCKS );
	while ( tot_blk ) {
		apedec->GetData( (char*)outbuf, BLOCKS_PER_DECODE, &blk_count );
		if ( !blk_count ) break;

  		write( dsp_fd, outbuf, blk_count * blk_align );
		tot_blk -= blk_count;
	}

	delete apedec;
	free( outbuf );
  }

  close( dsp_fd );

  return 0;
}

