#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <wavpack/wavpack.h>

int dsp_fd;
int channels;
short *outbuf;
char *filename;

int main( int argc, char **argv ) {
  int rc, idx;

  if ( argc < 2 ) {
  	puts( "Usage: wpkplay <filename.wv> ..." );
	return 1;
  }

  dsp_fd = open( "/dev/dsp", O_WRONLY );
  if ( dsp_fd == -1 ) {
	perror( "open /dev/dsp" );
	return 1;
  }
// two channels at most
  outbuf = (short*) malloc( 4096 * 2 * sizeof(int) );

  for ( idx = 1; idx < argc; idx++ ) {
	filename = argv[idx];

	printf( "Playing stream from %s...\n", filename );

	WavpackContext *wpc;
	char errbuf[80];
	wpc = WavpackOpenFileInput( filename, errbuf, OPEN_2CH_MAX, 0 );
	if ( !wpc ) {
		puts( "failed to open file" );
		continue;
	}

	int val = WavpackGetBitsPerSample( wpc );
	if ( val == 16 )
		val = AFMT_S16_LE;
	else {
	  	printf( "%d bits per sample format not supported\n", val );
		exit( 1 );
	}

	ioctl( dsp_fd, SNDCTL_DSP_SETFMT, &val );

	val = WavpackGetNumChannels( wpc );
	channels = val;
	if ( val == 1 )
	  	val = 0;
	else if ( val == 2 )
		val = 1;
	else {
  		printf( "%d channels format not supported\n", val );
		exit( 1 );
	}

	ioctl( dsp_fd, SNDCTL_DSP_STEREO, &val );

	val = WavpackGetSampleRate( wpc );
	ioctl( dsp_fd, SNDCTL_DSP_SPEED, &val );

	int smp_count;
	for (;;) {
		smp_count = WavpackUnpackSamples( wpc, (int32_t*)outbuf, 4096 );
		if ( !smp_count ) break;

		int len = smp_count * channels;
		for ( int i = 0; i < len; i++ )
			outbuf[i] = ((int32_t*)outbuf)[i];

  		write( dsp_fd, outbuf, smp_count * sizeof(short) * channels );
	}

	WavpackCloseFile( wpc );
  }

  free( outbuf );
  close( dsp_fd );

  return 0;
}

